/***************************************************************************
 *    GOnOffMaker.hpp - Abstract virtual base class for ON-OFF data        *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GOnOffMaker.hpp
 * @brief Abstract base class interface for ON-OFF data preparation
 * @author Pierrick Martin
 */

#ifndef GONOFFMAKER_HPP
#define GONOFFMAKER_HPP

/* __ Includes ___________________________________________________________ */
#include <vector>
#include <string>
#include "GBase.hpp"

/* __ Forward declarations _______________________________________________ */
class GOnOffBin;
class GModelPar;
class GObservations;

/***********************************************************************//**
* @class GOnOffMaker
*
* @brief Abstract base class interface for ON-OFF data preparation
*
* This class defines the interface for the preparation of ON and OFF data 
* for data analysis in the way adopted by existing Cherenkov telescopes 
* such as HESS, VERITAS, and MAGIC. 
*
* The different methods are implemented as derived classes such as
* GOnOffMakerReflected or GOnOffMakerRing. Each derived class can either take
* GSkyRegion objects as parameters for the definition of ON and OFF 
* regions, or generate them for a specified pattern.
*
* The parameters for the regions definition are stored in variable m_pars
* and accessible through operator[...]. They are GModelPar objects but not
* intended to be fitted, just because the GModelPar class offers a convenient
* way to handle them.
*
* The main output is an array of GOnOffBin elements (spectrum, map, cube),
* stored in the m_data variable, accessible through the data() method.
*
**************************************************************************/
class GOnOffMaker : public GBase {
	
public:
    // Constructors and destructors
    GOnOffMaker(void);
    GOnOffMaker(const GOnOffMaker& maker);
	explicit GOnOffMaker(const GObservations& obs);
	explicit GOnOffMaker(const std::string& filename);
    virtual ~GOnOffMaker(void);
	
    // Operators
    virtual GOnOffMaker&           operator=(const GOnOffMaker& maker);
	virtual GModelPar&             operator[](const int& index);
    virtual const GModelPar&       operator[](const int& index) const;
    virtual GModelPar&             operator[](const std::string& name);
    virtual const GModelPar&       operator[](const std::string& name) const;
  	
    // Pure virtual methods
    virtual void                   clear(void) = 0;
    virtual GOnOffMaker*           clone(void) const = 0;
    virtual std::string            type(void) const = 0;
    virtual std::string            print(const GChatter& chatter = NORMAL) const = 0;
	
    // Implemented methods
    const std::vector<GOnOffBin*>  data(void) const;
	const GObservations*           obs(void) const;
	void                           obs(const GObservations& obs);
	
protected:
    // Protected methods
    void         init_members(void);
    void         copy_members(const GOnOffMaker& maker);
    void         free_members(void);
	
    // Protected members
    std::string               m_type;         //!< Type of background evaluation
	GObservations*            m_obs;          //!< Container of observations
	std::vector<GOnOffBin*>   m_data;         //!< List of ON-OFF data
	std::vector<GModelPar*>   m_pars;         //!< Parameter pointers
};

#endif /* GONOFFMAKER_HPP */

																		  
																
																		
															
